==> apptics_flutter-0.0.1.tar.gz: android/src/main/kotlin/com/example/apptics_flutter/AppticsFlutterPlugin.kt <==
package com.example.apptics_flutter

import android.app.Activity
import android.util.Log
import com.zoho.apptics.analytics.AppticsAnalytics
import com.zoho.apptics.analytics.AppticsAnalytics.showReviewTrackingSettingsPopup
import com.zoho.apptics.analytics.AppticsEvents
import com.zoho.apptics.analytics.AppticsScreenTracker
import com.zoho.apptics.common.AppticsSettings
import com.zoho.apptics.common.AppticsSettings.setLocale
import com.zoho.apptics.common.AppticsSettings.setTrackingStatus
import com.zoho.apptics.common.AppticsTrackingState
import com.zoho.apptics.common.AppticsUser
import com.zoho.apptics.crash.AppticsCrashTracker
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import org.json.JSONObject
import java.util.*

const val TAG = "flutter_apptics_debug"

/** AppticsFlutterPlugin */
class AppticsFlutterPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity

    private lateinit var channel: MethodChannel
    private var currentActivity: Activity? = null

    override fun onAttachedToEngine(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "apptics_flutter")
        channel.setMethodCallHandler(this)
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }


    // Start
    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        currentActivity = binding.activity

    }

    override fun onDetachedFromActivityForConfigChanges() {

    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    override fun onDetachedFromActivity() {
        currentActivity = null
    }

    // End

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "getPlatformVersion" -> {
                getPlatformVersion(result)
            }
            "setUser" -> {
                setCurrentUser(call, result)
            }
            "isUserLoggedIn" -> {
                isUserLoggedIn(result)
            }
            "setDefaultLanguage" -> {
                setDefaultLanguage(call, result)
            }
            "setTrackingState" -> {
                setTrackingStatus(call, result)
            }
            "getTrackingState" -> {
                getTrackingStatus(result)
            }
            "openPrivacySettings" -> {
                openSettings(result)
            }
            "addEvent" -> {
                addEvent(call, result)
            }
            "screenAttached" -> {
                trackScreenAttached(call, result)
            }
            "screenDetached" -> {
                trackScreenDetached(call, result)
            }
            "showLastSessionCrashedPopup" -> {
                showLastSessionCrashedPopup(result)
            }
            "presentPrivacyReviewPopup" -> {
                showPrivacyReviewPopup(result)
            }
            "setCrashCustomProperty" -> {
                setCustomProperties(call, result)
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    private fun setDefaultLanguage(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, String>
        val language = arg["lang"]
        val array = language?.split(Regex("[-_]"))
        val locale: Locale? = array?.let { list ->
            when (list.size) {
                2 -> Locale(list[0], list[1])
                3 -> Locale(list[0], list[1], list[2])
                else -> {
                    Locale(list[0])
                }
            }
        }
        result.success(locale?.let { setLocale(it).toString() })
    }

    private fun getPlatformVersion(result: Result) {
        result.success("Android ${android.os.Build.VERSION.RELEASE}")

    }

    private fun setCustomProperties(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, Map<String, String>>
        val json = arg["properties"]
        if (json == null) {
            result.success(false)
        } else {
            AppticsCrashTracker.customProperties = JSONObject(json)
            Log.d(TAG, "setCustomProperties: ${AppticsCrashTracker.customProperties}")
            result.success(true)
        }
    }

    private fun showPrivacyReviewPopup(result: Result) {
        currentActivity?.let {
            showReviewTrackingSettingsPopup(it)
            result.success(true)
        }
    }

    private fun showLastSessionCrashedPopup(result: Result) {
        currentActivity?.let {
            result.success(AppticsCrashTracker.showLastSessionCrashedPopup(it, true).toString())
        }
    }

    private fun trackScreenAttached(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, String>
        val screenName: String = arg["screen_name"] ?: ""
        println(screenName)
        AppticsScreenTracker.inScreen(screenName)
        result.success(true)
    }

    private fun trackScreenDetached(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, String>
        val screenName: String = arg["screen_name"] ?: ""
        println(screenName)
        AppticsScreenTracker.outScreen(screenName)
        result.success(true)
    }


    private fun addEvent(call: MethodCall, result: Result) {
        val eventArg = call.arguments as Map<*, *>
        val eventName = eventArg["event"].toString()
        val groupName = eventArg["group"].toString()
        val customPropertiesMap = eventArg["properties"] as HashMap<String, String>
        Log.d(TAG, "addEvent: $eventName $groupName $customPropertiesMap")
        AppticsEvents.addEvent(eventName, groupName, customPropertiesMap)
        result.success(true)
    }

    private fun openSettings(result: Result) {
        if (currentActivity != null) {
            AppticsAnalytics.openSettings(currentActivity!!)
            result.success(true)
        } else {
            println("Null ${this.currentActivity}")
            result.success(false)
        }
    }

    private fun getTrackingStatus(result: Result) {
        result.success(AppticsSettings.getTrackingStatus().value)
    }

    private fun setTrackingStatus(call: MethodCall, result: Result) {
        val trackingState = call.arguments as Map<String, Int>
        val status = trackingState["status"]
        for (state in AppticsTrackingState.values()) {
            if (state.value == status) {
                result.success(setTrackingStatus(state))
            }
        }
    }

    private fun isUserLoggedIn(result: Result) {
        val isUserLoggedIn = AppticsUser.getCurrentUser() != null
        result.success(isUserLoggedIn)
    }

    private fun setCurrentUser(call: MethodCall, result: Result) {
        val userIdArg = call.arguments as Map<String, String>
        val userId: String? = userIdArg["userId"]
        result.success(userId?.let { AppticsUser.setUser(it) }.toString())
    }


}

==> apptics_flutter-0.0.2.tar.gz: android/src/main/kotlin/com/example/apptics_flutter/AppticsFlutterPlugin.kt <==
package com.example.apptics_flutter

import android.app.Activity
import android.content.Intent
import android.util.Log
import com.zoho.apptics.analytics.AppticsAnalytics
import com.zoho.apptics.analytics.AppticsAnalytics.showReviewTrackingSettingsPopup
import com.zoho.apptics.analytics.AppticsEvents
import com.zoho.apptics.analytics.AppticsScreenTracker
import com.zoho.apptics.appupdates.AppticsInAppUpdates
import com.zoho.apptics.common.AppticsSettings
import com.zoho.apptics.common.AppticsSettings.setLocale
import com.zoho.apptics.common.AppticsSettings.setTrackingStatus
import com.zoho.apptics.common.AppticsTrackingState
import com.zoho.apptics.common.AppticsUser
import com.zoho.apptics.crash.AppticsCrashTracker
import io.flutter.embedding.android.FlutterActivity
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import org.json.JSONObject
import java.util.*

const val TAG = "flutter_apptics_debug"

/** AppticsFlutterPlugin */
class AppticsFlutterPlugin : FlutterPlugin, MethodCallHandler, PluginRegistry.ActivityResultListener, ActivityAware {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity

    private lateinit var channel: MethodChannel
    private var currentActivity: Activity? = null

    override fun onAttachedToEngine(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "apptics_flutter")
        channel.setMethodCallHandler(this)
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }


    // Start
    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        currentActivity = binding.activity

    }

    override fun onDetachedFromActivityForConfigChanges() {

    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    override fun onDetachedFromActivity() {
        currentActivity = null
    }

    // End

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "getPlatformVersion" -> {
                getPlatformVersion(result)
            }

            "setUser" -> {
                setCurrentUser(call, result)
            }

            "isUserLoggedIn" -> {
                isUserLoggedIn(result)
            }

            "setDefaultLanguage" -> {
                setDefaultLanguage(call, result)
            }

            "setTrackingState" -> {
                setTrackingStatus(call, result)
            }

            "getTrackingState" -> {
                getTrackingStatus(result)
            }

            "openPrivacySettings" -> {
                openSettings(result)
            }

            "addEvent" -> {
                addEvent(call, result)
            }

            "screenAttached" -> {
                trackScreenAttached(call, result)
            }

            "screenDetached" -> {
                trackScreenDetached(call, result)
            }

            "showLastSessionCrashedPopup" -> {
                showLastSessionCrashedPopup(result)
            }

            "presentPrivacyReviewPopup" -> {
                showPrivacyReviewPopup(result)
            }

            "setCrashCustomProperty" -> {
                setCustomProperties(call, result)
            }

            "checkAndUpdateAlert" -> {
                checkAndUpdateAlert(result)
            }

            "onClickUpdate" -> {
                onClickUpdate(result)
            }

            "onClickReminder" -> {
                onClickReminder(result)
            }

            "onClickIgnore" -> {
                onClickIgnore(result)
            }

            "onClickNonSupportAlert" -> {
                onClickNonSupportAlert(result)
            }

            "onSendImpressionStatus" -> {
                onSendImpressionStatus(call, result)
            }

            else -> {
                result.notImplemented()
            }
        }
    }


    private fun setDefaultLanguage(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, String>
        val language = arg["lang"]
        val array = language?.split(Regex("[-_]"))
        val locale: Locale? = array?.let { list ->
            when (list.size) {
                2 -> Locale(list[0], list[1])
                3 -> Locale(list[0], list[1], list[2])
                else -> {
                    Locale(list[0])
                }
            }
        }
        result.success(locale?.let { setLocale(it).toString() })
    }

    private fun getPlatformVersion(result: Result) {
        result.success("Android ${android.os.Build.VERSION.RELEASE}")

    }

    private fun setCustomProperties(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, Map<String, String>>
        val json = arg["properties"]
        if (json == null) {
            result.success(false)
        } else {
            AppticsCrashTracker.customProperties = JSONObject(json)
            Log.d(TAG, "setCustomProperties: ${AppticsCrashTracker.customProperties}")
            result.success(true)
        }
    }

    private fun showPrivacyReviewPopup(result: Result) {
        currentActivity?.let {
            showReviewTrackingSettingsPopup(it)
            result.success(true)
        }
    }

    private fun showLastSessionCrashedPopup(result: Result) {
        currentActivity?.let {
            result.success(AppticsCrashTracker.showLastSessionCrashedPopup(it, true).toString())
        }
    }

    private fun trackScreenAttached(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, String>
        val screenName: String = arg["screen_name"] ?: ""
        println(screenName)
        AppticsScreenTracker.inScreen(screenName)
        result.success(true)
    }

    private fun trackScreenDetached(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, String>
        val screenName: String = arg["screen_name"] ?: ""
        println(screenName)
        AppticsScreenTracker.outScreen(screenName)
        result.success(true)
    }


    private fun addEvent(call: MethodCall, result: Result) {
        val eventArg = call.arguments as Map<*, *>
        val eventName = eventArg["event"].toString()
        val groupName = eventArg["group"].toString()
        val customPropertiesMap = eventArg["properties"] as HashMap<String, String>
        Log.d(TAG, "addEvent: $eventName $groupName $customPropertiesMap")
        AppticsEvents.addEvent(eventName, groupName, customPropertiesMap)
        result.success(true)
    }

    private fun openSettings(result: Result) {
        if (currentActivity != null) {
            AppticsAnalytics.openSettings(currentActivity!!)
            result.success(true)
        } else {
            println("Null ${this.currentActivity}")
            result.success(false)
        }
    }

    private fun getTrackingStatus(result: Result) {
        result.success(AppticsSettings.getTrackingStatus().value)
    }

    private fun setTrackingStatus(call: MethodCall, result: Result) {
        val trackingState = call.arguments as Map<String, Int>
        val status = trackingState["status"]
        for (state in AppticsTrackingState.values()) {
            if (state.value == status) {
                result.success(setTrackingStatus(state))
            }
        }
    }

    private fun isUserLoggedIn(result: Result) {
        val isUserLoggedIn = AppticsUser.getCurrentUser() != null
        result.success(isUserLoggedIn)
    }

    private fun setCurrentUser(call: MethodCall, result: Result) {
        val userIdArg = call.arguments as Map<String, String>
        val userId: String? = userIdArg["userId"]
        result.success(userId?.let { AppticsUser.setUser(it) }.toString())
    }

    private fun checkAndUpdateAlert(result: Result) {
        CoroutineScope(Dispatchers.IO).launch {
            val data = AppticsInAppUpdates.getUpdateDataIfAllConditionsSatisfied(currentActivity as FlutterActivity)
            result.success(data)
        }
    }

    private fun onClickUpdate(result: Result) {
        AppticsInAppUpdates.onClickUpdate(currentActivity as FlutterActivity)
        result.success(true)
    }

    private fun onClickReminder(result: Result) {
        AppticsInAppUpdates.onClickReminder()
        result.success(true)
    }

    private fun onClickIgnore(result: Result) {
        AppticsInAppUpdates.onClickIgnore()
        result.success(true)
    }

    private fun onClickNonSupportAlert(result: Result) {
        AppticsInAppUpdates.onClickNonSupportAlert()
        result.success(true)
    }

    private fun onSendImpressionStatus(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, String>
        val updateId = arg["updateId"]
        if (updateId != null) {
            AppticsInAppUpdates.onSendImpressionStatus(updateId)
        }
        result.success(true)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        AppticsInAppUpdates.doOnActivityResult(resultCode, resultCode)
        return false
    }

}
